import 'package:sql_query/database_config.dart';
import 'package:sql_query/database_model.dart';

class DB {
  static String? currentTableName;
  static Future<List<Map<String, dynamic>>> query(String tableName, {required DatabaseQuery databaseQuery}) async {
    List<Map<String, dynamic>> result = [];
    if (databaseQuery.get != null) {
      currentTableName = tableName;
      final select = databaseQuery.get?.select.toString().replaceAll('[', '').replaceAll(']', '').replaceAll('{NO_SELECTED_TABLE_NAME}', tableName);
      String closeParameter = '';

      final groupBy = databaseQuery.get?.groupBy.toString().replaceAll('[', '').replaceAll(']', '');
      if (groupBy != 'null') {
        closeParameter += ' GROUP BY $groupBy';
      }

      print(groupBy);

      if (databaseQuery.get?.orderBy != null) {
        // if (closeParameter != '') {
        //   closeParameter += ', ';
        // }
        closeParameter += ' ORDER BY ' + OrderBy.queryBuilder(databaseQuery.get!.orderBy!) + ' ';
      }

      if (databaseQuery.get?.limit != null) {
        closeParameter += ' LIMIT ${databaseQuery.get?.limit}';
      }
      result = await DatabaseConfig.get(query: 'SELECT ${select == 'null' ? '*' : select} FROM $tableName ' + (databaseQuery.get!.joinQuery).replaceAll('{NO_SELECTED_TABLE_NAME}', tableName) + Transaction.query(databaseQuery.get!) + closeParameter);
    } else if (databaseQuery.delete != null) {
      final isWhereNull = (databaseQuery.delete!.where == null
          ? null
          : databaseQuery.delete!.where!.isEmpty
              ? null
              : Transaction.query(databaseQuery.delete!).replaceAll('WHERE', ""));
      final res = await DatabaseConfig.delete(where: isWhereNull, tableName: tableName);
      if (res) {
        result.add({'status': 'success', 'message': 'data has been deleted'});
      }
    } else if (databaseQuery.insert != null) {
      final res = await DatabaseConfig.insert(data: databaseQuery.insert?.listData ?? databaseQuery.insert?.data, tableName: tableName);
      if (res) {
        result.add({'status': 'success', 'message': 'data has been added'});
      }
    } else if (databaseQuery.update != null) {
      final res = await DatabaseConfig.update(data: databaseQuery.update?.data ?? databaseQuery.insert?.listData, where: Transaction.query(databaseQuery.update!).replaceAll('WHERE', ""), tableName: tableName);
      if (res) {
        result.add({'status': 'success', 'message': 'data has been updated'});
      }
    }
    return result;
  }

  static init({required List<TableDatabase> tableList, String? databaseName}) async {
    List<String> insertList = [];
    for (final tl in tableList) {
      String insert = 'CREATE TABLE ${tl.tableName} (';
      int total = 0;
      for (final tc in tl.column) {
        ++total;
        String prop = '';

        if (tc.typeData == DataType.text) {
          prop += ' TEXT';
        } else if (tc.typeData == DataType.integer) {
          // if (tc.typeData == DataType.bool) {
          //   _boolData.add(tc.name);
          // }
          prop += ' INTEGER';
        } else if (tc.typeData == DataType.double) {
          prop += ' REAL';
        }

        if (tc.primaryKey) {
          prop += ' PRIMARY KEY';
        }

        if (tc.autoIncrement) {
          prop += ' AUTOINCREMENT';
        }

        if (tc.isNull) {
          prop += ' NULL';
        }

        insert += '${tc.name} ' + prop;
        if (tl.column.length > 1 && total != tl.column.length) {
          insert += ', ';
        }
      }
      insert += ')';
      insertList.add(insert);
    }
    try {
      DatabaseConfig.init(tables: insertList, dbName: databaseName);
    } catch (_) {
      print(_);
    }
  }

  static deleteDB() {
    DatabaseConfig.dbDelete();
  }

  static closeDB() {
    DatabaseConfig.dbClose();
  }
}
